/*
 * Copyright (c) 2004 Rocketalc LLC.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "includes.h"
#include "xmalloc.h"
#include "packet.h"
#include "log.h"
#include "auth.h"
#include "monitor_wrap.h"
#include "servconf.h"

/* import */
extern ServerOptions options;

static int
userauth_authd(Authctxt *authctxt)
{
	credentials ucred;
	signature usig;
	char *sigdata;
	int siglen;
	int authenticated = 0;
	if (!authctxt->valid) {
		debug2("userauth_authd: disabled (user invalid)");
		return 0;
	}
	ucred.uid = packet_get_int ();
	ucred.gid = packet_get_int ();
	/* XXX not really integers...fix this */
	ucred.valid_from = (time_t)packet_get_int ();
	ucred.valid_to = (time_t)packet_get_int ();
	sigdata = packet_get_string (&siglen);
	packet_check_eom ();
	memcpy (usig.data, sigdata, AUTH_RSA_SIGLEN);
	if (ucred.uid != authctxt->pw->pw_uid) 
		logit ("User id match required for authd authentication.");
	else {
		if(!use_privsep)
			authenticated = (auth_verify_signature (&ucred, &usig) == AUTH_OK);
		else
			authenticated = PRIVSEP (auth_verify_signature (&ucred, &usig));
	}
	memset (sigdata, 0, strlen(sigdata));
	xfree (sigdata);
	return authenticated;
}

Authmethod method_authd = {
	"authd",
	userauth_authd,
	&options.authd_authentication
};
